import os
import pickle

import mediapipe as mp
import cv2
import matplotlib.pyplot as plt


# Initialisation des outils de Mediapipe pour la détection des mains
mp_hands = mp.solutions.hands  # Module Mediapipe Hands pour la détection des mains
mp_drawing = mp.solutions.drawing_utils  # Utilitaire pour dessiner les landmarks des mains
mp_drawing_styles = mp.solutions.drawing_styles  # Styles pour afficher les landmarks des mains

hands = mp_hands.Hands(static_image_mode=True, min_detection_confidence=0.3)

# Répertoire contenant les images collectées
DATA_DIR = './data'

# Initialisation des listes pour stocker les données et les labels
data = []  # Contiendra les caractéristiques extraites des images
labels = []  # Contiendra les étiquettes associées aux images

# Parcours de chaque classe de gestes (répertoires contenant les images)
for dir_ in os.listdir(DATA_DIR):
    for img_path in os.listdir(os.path.join(DATA_DIR, dir_)):  # Parcours de toutes les images d'une classe
        data_aux = []  # Liste temporaire pour stocker les caractéristiques d'une image
        x_ = []  # Liste pour stocker les coordonnées X des landmarks
        y_ = []  # Liste pour stocker les coordonnées Y des landmarks

        # Lecture de l'image et conversion en format RGB (Mediapipe nécessite le format RGB)
        img = cv2.imread(os.path.join(DATA_DIR, dir_, img_path))
        img_rgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)

        # Détection de la main dans l'image
        results = hands.process(img_rgb)

        # Vérification si une main a bien été détectée
        if results.multi_hand_landmarks:
            for hand_landmarks in results.multi_hand_landmarks:
                # Extraction des coordonnées X et Y de chaque point de repère de la main
                for i in range(len(hand_landmarks.landmark)):
                    x = hand_landmarks.landmark[i].x
                    y = hand_landmarks.landmark[i].y

                    x_.append(x)
                    y_.append(y)

                # Normalisation des coordonnées : soustraction du minimum pour centrer les données
                for i in range(len(hand_landmarks.landmark)):
                    x = hand_landmarks.landmark[i].x
                    y = hand_landmarks.landmark[i].y
                    data_aux.append(x - min(x_))  # Normalisation X
                    data_aux.append(y - min(y_))  # Normalisation Y

            # Ajout des caractéristiques extraites et de l'étiquette associée à l'image
            data.append(data_aux)
            labels.append(dir_)

f = open('data.pickle', 'wb')
pickle.dump({'data': data, 'labels': labels}, f)
f.close()
