import pickle
import numpy as np
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score

# Charger les données depuis le fichier pickle
try:
    with open('./data.pickle', 'rb') as f:
        data_dict = pickle.load(f)
except FileNotFoundError:
    print("Erreur : Le fichier 'data.pickle' est introuvable. Assurez-vous de l'avoir généré.")
    exit()
except Exception as e:
    print(f"Erreur lors du chargement des données : {e}")
    exit()

# Vérifier le format des données
data = data_dict.get('data', None)
labels = data_dict.get('labels', None)

if data is None or labels is None:
    print("Erreur : Les données ou les labels sont manquants dans le fichier pickle.")
    exit()

# Vérifier les dimensions des données
try:
    data = np.array(data, dtype=object)  # Garder le type object pour gérer les dimensions variables
    max_length = max(len(item) for item in data)
    data = np.array([np.pad(item, (0, max_length - len(item)), mode='constant') for item in data])
    labels = np.array(labels)
except Exception as e:
    print(f"Erreur lors du traitement des données : {e}")
    exit()

# Vérifier les formes des tableaux après la conversion
print(f"Shape des données : {data.shape}")
print(f"Shape des labels : {labels.shape}")

# Division des données en train et test
x_train, x_test, y_train, y_test = train_test_split(data, labels, test_size=0.2, shuffle=True, stratify=labels)

# Initialisation et entraînement du modèle
model = RandomForestClassifier(n_estimators=100, random_state=42)
model.fit(x_train, y_train)

# Prédictions et évaluation
y_predict = model.predict(x_test)
score = accuracy_score(y_test, y_predict)

print(f"{score * 100:.2f}% des échantillons ont été classifiés correctement !")

# Sauvegarde du modèle
with open('model.p', 'wb') as f:
    pickle.dump({'model': model}, f)

print("Modèle enregistré sous 'model.p'.")
